#!/usr/bin/env python3
from tmss.wsgi import application  # required to set up django, even though not explicitly used
from tmss.tmssapp import models

from lofar.common import dbcredentials

import logging
import datetime
import pymysql
from optparse import OptionParser

logger = logging.getLogger(__file__)

def _execute_query(query, data=None):
    try:
        db = pymysql.connect(dbcreds.host, dbcreds.user, dbcreds.password, dbcreds.database)
        cursor = db.cursor(pymysql.cursors.DictCursor)
        cursor.execute(query, data)
        return cursor.fetchall()

    except Exception as e:
        logger.error("Could not execute query! %s" % e)


def query_project_details_from_momdb():
    """
    Queries MoM database for project details and returns the list of results
    :return: list of details as dict
    """
    logger.info("Querying MoM database for projects")
    query = """SELECT project.priority,
                      project.allowtriggers,
                      mom2object.name,
                      mom2object.description,
                      mom2object.mom2id
                FROM project
                JOIN mom2object ON project.mom2objectid=mom2object.id
                ORDER BY mom2id;
                """

    results = _execute_query(query)

    # dummy data:
    # results = [{"mom2id": 42,
    #             "name": "dummyproject",
    #             "description": "fake description",
    #             "priority": 1234,
    #             "allowtriggers": True}]

    return results


def get_project_details_from_momdb():
    """
    Obtains project details from MoM database and translates it into details as understood by the tmss data model.
    :return: dict mom2id -> project details as dict
    """
    logger.info("Getting project details from MoM database")
    mom_results = query_project_details_from_momdb()
    results = {}

    for mom_details in mom_results:

        # create new tmss details based on MoM details
        details = {"name": mom_details['name'],
                   "description": mom_details['description'],
                   "tags": ["migrated_from_MoM"],
                   "priority": mom_details['priority'],
                   "can_trigger": mom_details['allowtriggers'],
                   "private_data": True  # todo: check project.releasedate and compare to now or how to determine???
                   }

        # alterations to comply with constraints:
        if details['description'] is None:
            details['description'] = ''

        # add to return dict
        results[mom_details['mom2id']] = details

    return results


def query_subtask_details_for_project_from_momdb(project_mom2id):
    logger.info("Querying MoM database for tasks of project %s" % project_mom2id)
    # todo: double-check the correct use of ids. What refers to a mom2id and what refers to a database entry pk does not seem systematic and is very comfusing.
    query = '''SELECT mom2object.mom2id, mom2object.name, mom2object.description, mom2object.mom2objecttype, status.code,
                      lofar_pipeline.template, lofar_observation.default_template, lofar_pipeline.starttime, lofar_pipeline.endtime,
                      lofar_observation_specification.starttime AS obs_starttime, lofar_observation_specification.endtime AS obs_endtime
                      FROM mom2object
                      INNER JOIN mom2object AS ownerproject_mom2object ON mom2object.ownerprojectid = ownerproject_mom2object.id
                      INNER JOIN mom2objectstatus ON mom2object.currentstatusid = mom2objectstatus.id
                      INNER JOIN status ON mom2objectstatus.statusid = status.id
                      LEFT JOIN lofar_pipeline ON mom2object.id = lofar_pipeline.mom2objectid
                      LEFT JOIN lofar_observation ON mom2object.id = lofar_observation.mom2objectid
                      LEFT JOIN lofar_observation_specification ON lofar_observation.user_specification_id = lofar_observation_specification.id
                      WHERE ownerproject_mom2object.mom2id = %s
                      AND (mom2object.mom2objecttype = 'LOFAR_OBSERVATION' OR mom2object.mom2objecttype LIKE '%%PIPELINE%%');
                      '''

    parameters = (project_mom2id,)

    results = _execute_query(query, parameters)

    return results


def _dummy_subtask_template():
    try:
        return models.SubtaskTemplate.objects.get(name='dummy')
    except:
        dummy_template_details = {"name": "dummy",
                                  "description": 'Dummy Template',
                                  "version": '1',
                                  "schema": {},
                                  "realtime": False,
                                  "queue": False,
                                  "tags": ["DUMMY"]}

        return models.SubtaskTemplate.objects.create(**dummy_template_details)


def _dummy_scheduling_set(project):
    dummy_scheduling_set_details = {"name": 'dummy',
                            "description": "Dummy scheduling unit set",
                            "tags": ["DUMMY"],
                            "generator_doc": "{}",
                            "project": project,
                            "generator_template": None}


    return models.SchedulingSet.objects.create(**dummy_scheduling_set_details)


def _dummy_scheduling_unit_template():
    dummy_scheduling_unit_template_details = {"name": "dummy",
                                  "description": 'Dummy scheduling unit template',
                                  "version": 'v0.314159265359',
                                  "schema": {"mykey": "my value"},
                                  "tags": ["DUMMY"]}

    return models.RunTemplate.objects.create(**dummy_scheduling_unit_template_details)


def _dummy_scheduling_unit_draft(scheduling_set, template):
    dummy_scheduling_unit_draft_details = {"name": 'dummy',
                            "description": "Dummy scheduling_unit draft",
                            "tags": ["DUMMY"],
                            "requirements_doc": "{}",
                            "copy_reason": models.CopyReason.objects.get(value='template'),
                            "generator_instance_doc": "para",
                            "copies": None,
                            "scheduling_set": scheduling_set,
                            "generator_source": None,
                            "template": template}

    return models.RunDraft.objects.create(**dummy_scheduling_unit_draft_details)


def _dummy_scheduling_unit_blueprint(draft, template):

    dummy_scheduling_unit_blueprint_details = {"name": 'dummy',
                                   "description": "Dummy scheduling_unit blueprint",
                                   "tags": ["DUMMY"],
                                   "requirements_doc": "{}",
                                   "do_cancel": False,
                                   "draft": draft,
                                   "template": template}


    return  models.RunBlueprint.objects.create(**dummy_scheduling_unit_blueprint_details)


def _dummy_task_template():

    dummy_task_template_details = {"name": "dummy",
                                           "description": 'Dummy work request template',
                                           "validation_code_js": "",
                                           "version": 'v0.314159265359',
                                           "schema": {"mykey": "my value"},
                                           "tags": ["DUMMY"]}

    return models.TaskTemplate.objects.create(**dummy_task_template_details)


def _dummy_task_draft(scheduling_unit_draft, template):

    dummy_task_draft_details = {"name": 'dummy',
                                        "description": "Dummy work request draft",
                                        "tags": ["DUMMY"],
                                        "requirements_doc": "{}",
                                        "copy_reason": models.CopyReason.objects.get(value='template'),
                                        "copies": None,
                                        "scheduling_unit_draft": scheduling_unit_draft,
                                        "template": template}

    return models.TaskDraft.objects.create(**dummy_task_draft_details)


def _dummy_task_blueprint(draft, template, scheduling_unit_blueprint):

    dummy_task_blueprint_details = {"name": 'dummy',
                                            "description": "Dummy work request blueprint",
                                            "tags": ["DUMMY"],
                                            "requirements_doc": "{}",
                                            "do_cancel": False,
                                            "draft": draft,
                                            "template": template,
                                            "scheduling_unit_blueprint": scheduling_unit_blueprint}

    return models.TaskBlueprint.objects.create(**dummy_task_blueprint_details)


def get_subtask_details_from_momdb(project_mom2id, project):

    logger.info("Getting subtask details from MoM database")
    mom_results = query_subtask_details_for_project_from_momdb(project_mom2id)
    results = {}

    for mom_details in mom_results:

        # different types have some info in different spots, so they end up in different columns.
        # put same information in same spot to keep following code same for all tasks.
        # (maybe we want to instead separate these into different queries instead or union them in SQL?)

        if 'OBSERVATION' in mom_details['mom2objecttype']:
            type = models.SubtaskType.objects.get(value='observation')
            template_name = mom_details['default_template']
            start_time = mom_details['obs_starttime']
            end_time = mom_details['obs_endtime']
        elif 'PIPELINE' in mom_details['mom2objecttype']:
            type = models.SubtaskType.objects.get(value='pipeline')
            template_name = mom_details['template']
            start_time = mom_details['starttime']
            end_time = mom_details['endtime']
        else:
            logger.warning('Unknown type %(mom2objecttype)s' % mom_details)
            logger.warning('Skipping %s' % mom_details)
            continue

        # create new tmss details (leave out stuff that might go wrong)

        details = {"type": type,
                   "start_time": None,              # mandatory
                   "stop_time": None,               # mandatory
                   "state": None,                   # mandatory
                   "requested_state": None,         # mandatory
                   "specification": "{}",
                   "task_blueprint": None,  # optional, but required for project reference
                   "template": None,                # mandatory
                   "tags": ["migrated_from_MoM"]}

        # timestamps

        if start_time is not None:
            details['start_time'] = start_time
        else:
            details['start_time'] = datetime.datetime.utcfromtimestamp(0).isoformat() # not-null constraint

        if end_time is not None:
            details['stop_time'] = end_time
        else:
            details['stop_time'] = datetime.datetime.utcfromtimestamp(0).isoformat() # not-null constraint

        # state

        try:
            state = models.SubtaskState.objects.get(value=mom_details['code'])
            details['state'] = state
            details['requested_state'] = state
        except Exception as e:
            logger.error("No state choice matching '%s' in tmss database! %s" % (mom_details['code'], e))
            logger.warning('Skipping %s' % mom_details)
            continue

        # template

        if template_name is not None:
            try:
                details['template'] = models.SubtaskTemplate.objects.get(name=template_name)
            except Exception as e:
                logger.warning("No task template matching '%s' in tmss database! Using dummy instead! %s" % (template_name, e))

                # todo: create a lot of templates to reflect what was used for the actual task?
                # todo: raise Exception (or continue) once we have proper templates for everything.
                details["template"] = _dummy_subtask_template()

        else:
            logger.warning('Missing template name in MoM details!')
            logger.warning('Skipping %s' % mom_details)
            continue

        # ----------------
        # todo: the following entries are needed to relate a task to it's project.
        # todo: we should substitute the dummy items by items that reflect the actual task details

        # scheduling set
        scheduling_set = _dummy_scheduling_set(project)

        # scheduling unit template
        scheduling_unit_template = _dummy_scheduling_unit_template()

        # scheduling unit draft
        scheduling_unit_draft = _dummy_scheduling_unit_draft(scheduling_set, scheduling_unit_template)

        # scheduling unit blueprint
        scheduling_unit_blueprint = _dummy_scheduling_unit_blueprint(scheduling_unit_draft, scheduling_unit_template)

        # work request template
        task_template = _dummy_task_template()

        # work request draft
        task_draft = _dummy_task_draft(scheduling_unit_draft, task_template)

        # work request blueprint
        details['task_blueprint'] = _dummy_task_blueprint(task_draft,
                                                                          task_template,
                                                                          scheduling_unit_blueprint)
        # ----------------

        # add task mom2id and its details to return dict
        results[mom_details['mom2id']] = details

    return results


def main():
    project_details = get_project_details_from_momdb()

    for p_id, p_details in project_details.items():

        logger.info("---\nNow migrating project %s..." % p_details['name'])
        project = models.Project.objects.create(**p_details)
        logger.info("...created new project with tmss id %s" % project.id)

        task_details = get_subtask_details_from_momdb(p_id, project)
        for t_id, t_details in task_details.items():

            logger.info("...creating new task mom2id %s" % t_id)
            task = models.Subtask.objects.create(**t_details)
            logger.info("...created new task with tmss id %s" % task.id)

        logger.info("...done migrating project %s." % p_details['name'])


if __name__ == "__main__":
    logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
                        level=logging.DEBUG)

    # Check the invocation arguments
    parser = OptionParser("%prog [options]", description='run the mom to tmss db conversion script')
    parser.add_option_group(dbcredentials.options_group(parser))
    parser.set_defaults(dbcredentials="MoM")
    (options, args) = parser.parse_args()

    global dbcreds
    dbcreds = dbcredentials.parse_options(options)
    logger.info("Using dbcreds: %s", dbcreds.stringWithHiddenPassword())

    # note: this requires a config file .lofar/dbcredentials/mom.ini, with contents as such (adapt as needed):
    #
    # [database:MoM]
    # type = mysql
    # host = localhost
    # port = 3306
    # user = momuser
    # password = mompass
    # database = lofar_mom_test_tmss


    main()
