#!/usr/bin/env python3
import os
import sys
from glob import glob
import subprocess as sp
import logging


logger = logging.getLogger(__file__)
here = os.path.dirname(__file__)
relapath = '/tmss/tmssapp/migrations/'
template = """
from django.db import migrations

from lofar.sas.tmss.tmss.tmssapp.populate import *

class Migration(migrations.Migration):

    dependencies = [
        ('tmssapp', '%s'),
    ]

    # Start SubTask id with 2 000 000 to avoid overlap with 'old' (test/production) OTDB
    operations = [ migrations.RunSQL('ALTER SEQUENCE tmssapp_SubTask_id_seq RESTART WITH 2000000;'),
                   migrations.RunPython(populate_choices),
                   migrations.RunPython(populate_misc),
                   migrations.RunPython(populate_lofar_json_schemas) ]
"""


def execute_and_log(cmd):

    logger.info('COMMAND: %s' % cmd)
    p = sp.Popen(cmd, shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
    out, err = p.communicate()
    if out is not None:
        logger.info("STDOUT: %s" % out.decode('utf-8').strip())
    if err is not None:
        logger.info("STDERR: %s" % err.decode('utf-8').strip())


def delete_old_migrations():
    logger.info('Removing old migrations...')

    files = glob_migrations()
    for f in [path for path in files if ("auto" in path or "populate" in path)]:
        logger.info('Deleting: %s' % f)
        os.remove(f)


def make_django_migrations():

    logger.info('Making Django migrations...')
    execute_and_log('/usr/bin/env python3 %s/manage.py makemigrations' % here)


def make_populate_migration():

    logger.info('Making migration for populating database...')
    last_migration = determine_last_migration()
    migration = template % last_migration

    path = here + relapath + '%s_populate.py' % str(int(last_migration.split('_')[0])+1).zfill(4)
    logger.info('Writing to: %s' % path)
    with open(path,'w') as f:
        f.write(migration)


def glob_migrations():
    paths = glob(here + '/' + relapath + '0*_*')
    return paths


def determine_last_migration():
    logger.info('Determining last migration...')
    files = glob_migrations()
    files = [os.path.basename(path) for path in files]
    f = max(files)
    last_migration = f.split('.py')[0]
    logger.info('Determined last migration: %s' % last_migration)
    return last_migration


def put_migrations_under_version_control():
    logger.info('Putting migrations under version control...')
    files = glob_migrations()
    for f in files:
        execute_and_log('git add %s' % f)


def remake_migrations():
    delete_old_migrations()
    make_django_migrations()
    make_populate_migration()


if __name__ == "__main__":

    logger.setLevel(logging.DEBUG)

    handler = logging.StreamHandler()
    handler.setLevel(logging.INFO)
    logger.addHandler(handler)

    remake_migrations()
